@@ -8,6 +8,51 @@
 
 if ( preg_match('{/always.php$}', $_SERVER['SCRIPT_NAME'] ) ) header('Location: index.php');
 
+// XSS Protection
+function filter_post(&$val, $index) {
+    if(in_array($index, ["newpass1", "newpass2", "password"])) return;
+
+    switch (gettype($val)) {
+        case "string":
+            $val = htmlspecialchars($val);
+            break;
+
+        case "array":
+            array_walk_recursive($val, function(&$v) {
+                if (gettype($v) == "string") {
+                    $v = htmlspecialchars($v);
+                }
+            });
+            break;
+    }
+}
+
+function clean_get() {
+    $temp = [];
+
+    foreach($_GET as $key => $value) {
+        // XSS is possible in both key and values
+        $k = htmlspecialchars($key);
+        $v = htmlspecialchars($value);
+        $temp[$k] = $v;
+    }
+
+    return $temp;
+}
+
+// Before anything else is executed we filter all the user input, a lot of code in this project
+// relies on variables that are easily manipulated by the user. These lines and functions filter all those variables.
+if(isset($_POST)) array_walk($_POST, 'filter_post');
+$_GET = clean_get();
+if (isset($_SERVER['REQUEST_URI'])) {
+    $_SERVER['REQUEST_URI'] = str_replace("&amp;", "&", htmlspecialchars($_SERVER['REQUEST_URI']));
+}
+if (isset($_SERVER['HTTP_REFERER'])) {
+    $_SERVER['HTTP_REFERER'] = htmlspecialchars(@$_SERVER['HTTP_REFERER']);
+}
+
+
+
 // Ensure the configuration starts out as an empty object.
 $c = (object) array();
 $c->script_start_time = microtime(true);
@@ -38,17 +83,37 @@ function early_exception_handler($e) {
   foreach( $trace AS $k => $v ) {
     printf( "%s[%d] %s%s%s()\n", $v['file'], $v['line'], (isset($v['class'])?$v['class']:''), (isset($v['type'])?$v['type']:''), (isset($v['function'])?$v['function']:'') );
   }
-  
+  error_log("PHP Early Exception: [".$e->getCode()."] ".$e->getmessage()." at ".$e->getFile().":".$e->getLine());
 }
 set_exception_handler('early_exception_handler');
 
-$c->default_timezone = ini_get ( 'date.timezone' );
-if (empty ( $c->default_timezone )) {
-    if (isset ( $_SERVER ['HTTP_X_DAVICAL_TESTCASE'] )) {
-        $c->default_timezone = 'Pacific/Auckland';
+function early_catch_fatal_error() {
+  global $request;
+
+  if ( !empty($request) ) return;
+
+  // Getting Last Error
+  $e =  error_get_last();
+
+  // Check if Last error is of type FATAL
+  if (isset($e['type']) && $e['type'] == E_ERROR) {
+    if ( !headers_sent() ) {
+      header("Content-type: text/plain");
+      header( sprintf("HTTP/1.1 %d %s", 500, getStatusMessage(500)) );
     }
+    echo "PHP Fatal error: ".$e['message']."\n";
+    echo "At line ", $e['line'], " of ", $e['file'], "\n";
+    error_log("PHP Fatal Error: '".$e['message']. "' at line ". $e['line']. " of ". $e['file']);
+  }
+}
+register_shutdown_function('early_catch_fatal_error');
+
+$c->default_timezone = ini_get ( 'date.timezone' );
+if (isset ( $_SERVER ['HTTP_X_DAVICAL_TESTCASE'] ) ) {
+    $c->default_timezone = 'Pacific/Auckland';
+}
+else if (empty ( $c->default_timezone )) {
     $c->default_timezone = 'UTC';
-    date_default_timezone_set ( 'UTC' );
 }
 
 // Default some of the configurable values
@@ -62,17 +127,20 @@ $c->allow_get_email_visibility = false;
 $c->permission_scan_depth = 2;
 $c->expand_pdo_parameters = true;
 $c->home_calendar_name    = 'calendar';
+$c->home_out_calendar_name = '.out';
+$c->home_in_calendar_name = '.in';
 $c->home_addressbook_name = 'addresses';
+$c->carddav_max_resource_size = 6550000;
 $c->enable_row_linking = true;
 $c->enable_scheduling = false;
 $c->http_auth_mode = 'Basic';
 // $c->default_locale = array('es_MX', 'es_AR', 'es', 'pt');  // An array of locales to try, or just a single locale
-// $c->local_tzid = 'Pacific/Auckland';  // Perhaps we should read from /etc/timezone - I wonder how standard that is?
 $c->default_locale = 'en';
 $c->locale_path = '../locale';
 $c->base_url = preg_replace('#/[^/]+\.php.*$#', '', $_SERVER['SCRIPT_NAME']);
 $c->base_directory = preg_replace('#/[^/]*$#', '', $_SERVER['DOCUMENT_ROOT']);
 $c->default_privileges = array('read-free-busy', 'schedule-deliver');
+$c->list_everyone = true;
 
 $c->enable_auto_schedule = true;
 
@@ -94,6 +162,10 @@ $c->readonly_webdav_collections = true;    // WebDAV access is readonly
 // Kind of private configuration values
 $c->total_query_time = 0;
 
+// Any many times GetMoreInstances in inc/RRule.php should loop trying to
+// find more instances.
+$c->rrule_loop_limit = 100;
+
 $c->dbg = array();
 
 
@@ -129,22 +201,6 @@ if ( !isset($_SERVER['SERVER_NAME']) ) {
   @dbg_error_log( 'WARN', "Your webserver is not setting the SERVER_NAME parameter. You may need to set \$c->domain_name in your configuration.  Using IP address meanhwhile..." );
 }
 
-/**
-* Calculate the simplest form of reference to this page, excluding the PATH_INFO following the script name.
-*/
-$c->protocol_server_port = sprintf( '%s://%s%s',
-                 (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on'? 'https' : 'http'),
-                 $_SERVER['SERVER_NAME'],
-                 (
-                    ( (!isset($_SERVER['HTTPS']) || $_SERVER['HTTPS'] != 'on')
-                                        && (!isset($_SERVER['SERVER_PORT']) || $_SERVER['SERVER_PORT'] == 80) )
-                           || (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on'
-                           				&& (!isset($_SERVER['SERVER_PORT']) || $_SERVER['SERVER_PORT'] == 443) )
-                   ? ''
-                   : ':'.$_SERVER['SERVER_PORT']
-                 ) );
-$c->protocol_server_port_script = $c->protocol_server_port . ($_SERVER['SCRIPT_NAME'] == '/index.php' ? '' : $_SERVER['SCRIPT_NAME']);
-
 
 /**
 * We use @file_exists because things like open_basedir might noisily deny
@@ -177,6 +233,44 @@ else {
 $config_warnings = trim(ob_get_contents());
 ob_end_clean();
 
+/**
+* Override server-detected variables with those from X-Forwarded headers
+*/
+if ( isset($c->trust_x_forwarded) && $c->trust_x_forwarded ) {
+  if ( isset($_SERVER['HTTP_X_REAL_IP']) ) {
+    $_SERVER['REMOTE_ADDR'] = $_SERVER['HTTP_X_REAL_IP'];
+  } elseif ( isset($_SERVER['HTTP_X_FORWARDED_FOR']) ) {
+    list($_SERVER['REMOTE_ADDR'], $rest) = explode( ',', $_SERVER['HTTP_X_FORWARDED_FOR']);
+  }
+  if ( isset($_SERVER['HTTP_X_FORWARDED_PROTO']) ) {
+    if ($_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https') {
+      $_SERVER['HTTPS'] = 'on';
+    } else {
+      $_SERVER['HTTPS'] = 'off';
+    }
+  }
+  if ( isset($_SERVER['HTTP_X_FORWARDED_PORT']) ) {
+    $_SERVER['SERVER_PORT'] = $_SERVER['HTTP_X_FORWARDED_PORT'];
+  }
+}
+
+/**
+* Calculate the simplest form of reference to this page, excluding the PATH_INFO following the script name.
+*/
+$c->protocol_server_port = sprintf( '%s://%s%s',
+                 (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on'? 'https' : 'http'),
+                 $_SERVER['SERVER_NAME'],
+                 (
+                     ( (!isset($_SERVER['HTTPS']) || $_SERVER['HTTPS'] != 'on')
+                          && (!isset($_SERVER['SERVER_PORT']) || $_SERVER['SERVER_PORT'] == 80) )
+                     || ( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on'
+                          && (!isset($_SERVER['SERVER_PORT']) || $_SERVER['SERVER_PORT'] == 443) )
+                     ? ''
+                     : ':'.$_SERVER['SERVER_PORT']
+                 ) );
+$c->protocol_server_port_script = $c->protocol_server_port . ($_SERVER['SCRIPT_NAME'] == '/index.php' ? '' : $_SERVER['SCRIPT_NAME']);
+
+
 if ( !isset($c->page_title) ) $c->page_title = $c->system_name;
 
 if ( isset($_SERVER['HTTP_X_DAVICAL_TESTCASE']) ) {
@@ -203,8 +297,8 @@ if ( function_exists('awl_set_locale') ) {
 *
 */
 $c->code_version = 0;
-$c->want_awl_version = '0.55';
-$c->version_string = '1.1.3.1'; // The actual version # is replaced into that during the build /release process
+$c->want_awl_version = '0.64';
+$c->version_string = '1.1.12'; // The actual version # is replaced into that during the build /release process
 if ( isset($c->version_string) && preg_match( '/(\d+)\.(\d+)\.(\d+)(.*)/', $c->version_string, $matches) ) {
   $c->code_major = $matches[1];
   $c->code_minor = $matches[2];
@@ -221,7 +315,7 @@ $_SERVER['SERVER_NAME'] = $c->domain_name;
 
 require_once('AwlQuery.php');
 
-$c->want_dbversion = array(1,2,11);
+$c->want_dbversion = array(1,3,5);
 $c->schema_version = 0;
 $qry = new AwlQuery( 'SELECT schema_major, schema_minor, schema_patch FROM awl_db_revision ORDER BY schema_id DESC LIMIT 1;' );
 if ( $qry->Exec('always',__LINE__,__FILE__) && $row = $qry->Fetch() ) {
@@ -238,6 +332,7 @@ if ( $qry->Exec('always',__LINE__,__FILE__) && $row = $qry->Fetch() ) {
     $qry->QDo('SET TIMEZONE TO ?', $c->default_timezone );
   }
 }
+date_default_timezone_set ( $c->default_timezone );
 
 require_once('Principal.php');
 
@@ -302,6 +397,7 @@ function getStatusMessage($status) {
 * Construct a URL from the supplied dav_name.  The URL will be urlencoded,
 * except for any '/' characters in it.
 * @param string $partial_path  The part of the path after the script name
+* @param boolean $force_script  Whether to force-include caldav.php in the URL. Defaults to false.
 */
 function ConstructURL( $partial_path, $force_script = false ) {
   global $c;
@@ -325,6 +421,11 @@ function ConstructURL( $partial_path, $force_script = false ) {
     $url = str_replace( 'caldav.php/caldav.php', 'caldav.php', $url );  // Ensure we don't double any 'caldav.php/'
   }
 
+  // don't put the script name in special URLs, these would not be valid resources
+  $url = str_replace('/principals/users/caldav.php/', '/principals/users/', $url );
+  $url = str_replace('/principals/resources/caldav.php/', '/principals/resources/', $url );
+  $url = str_replace('/__uids__/caldav.php/', '/__uids__/', $url );
+
   return $url;
 }
 
@@ -332,12 +433,13 @@ function ConstructURL( $partial_path, $force_script = false ) {
 /**
 * Deconstruct a dav_name from the supplied URL.  The dav_name will be urldecoded.
 *
-* @param string $partial_path  The part of the path after the script name
+* @param string $url  The full path including the script name
 */
-function DeconstructURL( $url, $force_script = false ) {
+function DeconstructURL( $url ) {
   global $c;
 
   $dav_name = rawurldecode($url);
+  $dav_name = trim($dav_name);
 
   /** Allow a path like .../username/calendar.ics to translate into the calendar URL */
   if ( preg_match( '#^(/[^/]+/[^/]+).ics$#', $dav_name, $matches ) ) {
@@ -366,26 +468,8 @@ function DeconstructURL( $url, $force_script = false ) {
 * @param string $isodate The date to convert
 */
 function ISODateToHTTPDate( $isodate ) {
-  // It is necessary to use English for this, explicitly.  See Debian BTS Bug#661985 for more info.
-  $month = gmstrftime('%m', strtotime($isodate));
-  switch( intval($month) ) {
-  	case 1: $month = 'Jan'; break;
-  	case 2: $month = 'Feb'; break;
-  	case 3: $month = 'Mar'; break;
-  	case 4: $month = 'Apr'; break;
-  	case 5: $month = 'May'; break;
-  	case 6: $month = 'Jun'; break;
-  	case 7: $month = 'Jul'; break;
-  	case 8: $month = 'Aug'; break;
-  	case 9: $month = 'Sep'; break;
-  	case 10: $month = 'Oct'; break;
-  	case 11: $month = 'Nov'; break;
-  	case 12: $month = 'Dec'; break;
-  	default:
-  		throw new Exception('Invalid month '.$month);
-  }
   // Use strtotime since strptime is not available on Windows platform.
-  return( gmstrftime('%a, %d '.$month.' %Y %H:%M:%S GMT', strtotime($isodate)) );
+  return( gmdate('D, d M Y H:i:s T', strtotime($isodate)) );
 }
 
 /**
