@@ -107,11 +107,36 @@ class CalDAVRequest
   public $ticket;
 
   /**
-   * An array of values from the 'Prefer' header.  At present only 'return-minimal' is acted on in any way - you
+   * An array of values from the 'Prefer' header.  At present only 'return=minimal' is acted on in any way - you
    * can test that value with the PreferMinimal() method.
    */
   private $prefer;
-  
+
+  /* These fields were being added dynmically. I initially tried making them
+   * private, but many tests failed in odd ways. Having them public matches
+   * the existing behaviour.
+   */
+  public $path;
+  public $content_type;
+  public $overwrite;
+  public $collection;
+  public $user_no;
+  public $username;
+  public $by_email;
+  public $principal_id;
+  public $permissions;
+  public $_is_collection;
+  public $_is_principal;
+  public $_is_proxy_request;
+  public $_locks_found;
+  public $xml_tags;
+  public $etag_if_match;
+  public $etag_none_match;
+  public $proxy_type;
+  public $if_clause;
+  public $lock_token;
+  public $timeout;
+
   /**
   * Create a new CalDAVRequest object.
   */
@@ -125,10 +150,10 @@ class CalDAVRequest
       $this->prefer = explode( ',', $_SERVER['HTTP_PREFER']);
     }
     else if ( isset($_SERVER['HTTP_BRIEF']) && (strtoupper($_SERVER['HTTP_BRIEF']) == 'T') ) {
-      $this->prefer = array( 'return-minimal');
+      $this->prefer = array( 'return=minimal');
     }
-    else 
-      $this->prefer = array(); 
+    else
+      $this->prefer = array();
 
     /**
     * Our path is /<script name>/<user name>/<user controlled> if it ends in
@@ -149,11 +174,17 @@ class CalDAVRequest
     else {
       $this->path = '/';
       if ( isset($_SERVER['REQUEST_URI']) ) {
-        if ( preg_match( '{^(.*?\.php)(.*)$}', $_SERVER['REQUEST_URI'], $matches ) ) {
+        if ( preg_match( '{^(.*?\.php)([^?]*)}', $_SERVER['REQUEST_URI'], $matches ) ) {
           $this->path = $matches[2];
           if ( substr($this->path,0,1) != '/' )
             $this->path = '/'.$this->path;
         }
+        /* support php_fpm under Apache 2.4 */
+        else if ( isset($_SERVER['REQUEST_URI']) && isset($_SERVER['SCRIPT_NAME'] ) ) {
+          if ( preg_match( '{^(.*?\.php)([^?]*)}', $_SERVER['SCRIPT_NAME'] ) ) {
+            $this->path = $_SERVER['REQUEST_URI'];
+          }
+        }
         else if ( $_SERVER['REQUEST_URI'] != '/' ) {
           dbg_error_log('LOG', 'Server is not supplying PATH_INFO and REQUEST_URI does not include a PHP program.  Wildly guessing "/"!!!');
         }
@@ -217,9 +248,13 @@ class CalDAVRequest
       $_SERVER['REQUEST_METHOD'] = $_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'];
     }
     $this->method = $_SERVER['REQUEST_METHOD'];
-    $this->content_type = (isset($_SERVER['CONTENT_TYPE']) ? $_SERVER['CONTENT_TYPE'] : null);
-    if ( preg_match( '{^(\S+/\S+)\s*(;.*)?$}', $this->content_type, $matches ) ) {
-      $this->content_type = $matches[1];
+    if (isset($_SERVER['CONTENT_TYPE'])) {
+        $this->content_type = $_SERVER['CONTENT_TYPE'];
+        if ( preg_match( '{^(\S+/\S+?)\s*(;.*)?$}', $this->content_type, $matches ) ) {
+            $this->content_type = $matches[1];
+        }
+    } else {
+        $this->content_type = null;
     }
     if ( strlen($c->raw_post) > 0 ) {
       if ( $this->method == 'PROPFIND' || $this->method == 'REPORT' || $this->method == 'PROPPATCH' || $this->method == 'BIND' || $this->method == 'MKTICKET' || $this->method == 'ACL' ) {
@@ -269,7 +304,7 @@ class CalDAVRequest
     }
     if ( !is_int($this->depth) && "infinity" == $this->depth  ) $this->depth = DEPTH_INFINITY;
     $this->depth = intval($this->depth);
-    
+
     /**
     * MOVE/COPY use a "Destination" header and (optionally) an "Overwrite" one.
     */
@@ -377,7 +412,7 @@ EOSQL;
       $cache = getCacheInstance();
       $cache->delete( 'collection-'.$params[':dav_name'], null );
       $cache->delete( 'principal-'.$params[':parent_container'], null );
-      
+
       $qry = new AwlQuery( "SELECT * FROM collection WHERE dav_name = :dav_name", array( ':dav_name' => $matches[1] ) );
       if ( $qry->Exec('caldav',__LINE__,__FILE__) && $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
         $this->collection_id = $row->collection_id;
@@ -440,7 +475,6 @@ EOSQL;
     if ( $this->collection_type == 'principal' || $this->collection_type == 'email' || $this->collection_type == 'proxy' ) {
       $this->collection = $this->principal->AsCollection();
       if( $this->collection_type == 'proxy' ) {
-        $this->collection = $this->principal->AsCollection();
         $this->collection->is_proxy = 't';
         $this->collection->type = 'proxy';
         $this->collection->proxy_type = $this->proxy_type;
@@ -467,89 +501,6 @@ EOSQL;
     */
     $this->setPermissions();
 
-    $this->supported_methods = array(
-      'OPTIONS' => '',
-      'PROPFIND' => '',
-      'REPORT' => '',
-      'DELETE' => '',
-      'LOCK' => '',
-      'UNLOCK' => '',
-      'MOVE' => '',
-      'ACL' => ''
-    );
-    if ( $this->IsCollection() ) {
-      switch ( $this->collection_type ) {
-        case 'root':
-        case 'email':
-          // We just override the list completely here.
-          $this->supported_methods = array(
-            'OPTIONS' => '',
-            'PROPFIND' => '',
-            'REPORT' => ''
-          );
-          break;
-        case 'schedule-inbox':
-        case 'schedule-outbox':
-          $this->supported_methods = array_merge(
-            $this->supported_methods,
-            array(
-              'POST' => '', 'GET' => '', 'PUT' => '', 'HEAD' => '', 'PROPPATCH' => ''
-            )
-          );
-          break;
-        case 'calendar':
-          $this->supported_methods['GET'] = '';
-          $this->supported_methods['PUT'] = '';
-          $this->supported_methods['HEAD'] = '';
-          break;
-        case 'collection':
-        case 'principal':
-          $this->supported_methods['GET'] = '';
-          $this->supported_methods['PUT'] = '';
-          $this->supported_methods['HEAD'] = '';
-          $this->supported_methods['MKCOL'] = '';
-          $this->supported_methods['MKCALENDAR'] = '';
-          $this->supported_methods['PROPPATCH'] = '';
-          $this->supported_methods['BIND'] = '';
-          break;
-      }
-    }
-    else {
-      $this->supported_methods = array_merge(
-        $this->supported_methods,
-        array(
-          'GET' => '',
-          'HEAD' => '',
-          'PUT' => ''
-        )
-      );
-    }
-
-    $this->supported_reports = array(
-      'DAV::principal-property-search' => '',
-      'DAV::expand-property' => '',
-      'DAV::sync-collection' => ''
-    );
-    if ( isset($this->collection) && $this->collection->is_calendar ) {
-      $this->supported_reports = array_merge(
-        $this->supported_reports,
-        array(
-          'urn:ietf:params:xml:ns:caldav:calendar-query' => '',
-          'urn:ietf:params:xml:ns:caldav:calendar-multiget' => '',
-          'urn:ietf:params:xml:ns:caldav:free-busy-query' => ''
-        )
-      );
-    }
-    if ( isset($this->collection) && $this->collection->is_addressbook ) {
-      $this->supported_reports = array_merge(
-        $this->supported_reports,
-        array(
-          'urn:ietf:params:xml:ns:carddav:addressbook-query' => '',
-          'urn:ietf:params:xml:ns:carddav:addressbook-multiget' => ''
-        )
-      );
-    }
-
 
     /**
     * If the content we are receiving is XML then we parse it here.  RFC2518 says we
@@ -641,8 +592,10 @@ EOSQL;
         $params[':request_path'] = $this->path;
       }
       $qry = new AwlQuery( $sql, $params );
-      if ( $qry->Exec('caldav',__LINE__,__FILE__) && $permission_result = $qry->Fetch() )
-        $this->privileges |= bindec($permission_result->perm);
+      if ( $qry->Exec('caldav',__LINE__,__FILE__) && $permission_result = $qry->Fetch() ) {
+        $perm = $permission_result->perm;
+        if (isset($perm)) $this->privileges |= bindec($permission_result->perm);
+      }
 
       dbg_error_log( 'caldav', 'Restricted permissions for user accessing someone elses hierarchy: %s', decbin($this->privileges) );
       if ( isset($this->ticket) && $this->ticket->MatchesPath($this->path) ) {
@@ -719,7 +672,7 @@ EOSQL;
   }
 
 
-  private static function supportedPrivileges() { 
+  private static function supportedPrivileges() {
     return array(
       'all' => array(
         'read' => translate('Read the content of a resource or collection'),
@@ -749,7 +702,7 @@ EOSQL;
       )
     );
   }
-  
+
   /**
   * Returns the dav_name of the resource in our internal namespace
   */
@@ -927,12 +880,13 @@ EOSQL;
 
 
   /**
-   * Returns true if the 'Prefer: return-minimal' or 'Brief: t' were present in the request headers.
+   * Returns true if the 'Prefer: return=minimal' or 'Brief: t' were present in the request headers.
    */
   function PreferMinimal() {
     if ( empty($this->prefer) ) return false;
     foreach( $this->prefer AS $v ) {
-      if ( $v == 'return-minimal' ) return true;
+      if ( $v == 'return=minimal' ) return true;
+      if ( $v == 'return-minimal' ) return true; // RFC7240 up until draft -15 (Oct 2012)
     }
     return false;
   }
@@ -1117,13 +1071,13 @@ EOSQL;
 
   /**
    * Check that the incoming Etag matches the one for the existing (or non-existing) resource.
-   * 
+   *
    * @param boolean $exists Whether the destination exists
    * @param string $dest_etag The etag for the destination.
    */
   function CheckEtagMatch( $exists, $dest_etag ) {
     global $c;
-    
+
     if ( ! $exists ) {
       if ( (isset($this->etag_if_match) && $this->etag_if_match != '') ) {
         /**
@@ -1136,20 +1090,21 @@ EOSQL;
       }
     }
     else {
-    
+
       if ( isset($c->strict_etag_checking) && $c->strict_etag_checking )
          $trim_chars = '\'\\" ';
       else
         $trim_chars = ' ';
 
-      if ( isset($this->etag_if_match) && $this->etag_if_match != '' && trim( $this->etag_if_match, $trim_chars) != trim( $dest_etag, $trim_chars ) ) {
+      if ( isset($this->etag_if_match) && $this->etag_if_match != '' && $this->etag_if_match != '*'
+              && trim( $this->etag_if_match, $trim_chars) != trim( $dest_etag, $trim_chars ) ) {
         /**
         * RFC2068, 14.25:
         * If none of the entity tags match, or if "*" is given and no current
         * entity exists, the server MUST NOT perform the requested method, and
         * MUST return a 412 (Precondition Failed) response.
         */
-        $this->PreconditionFailed(412,'if-match',sprintf('Existing resource ETag of <<%s>> does not match <<%s>>', $dest_etag, $this->etag_if_match) );
+        $this->PreconditionFailed(412,'if-match',sprintf('Existing resource ETag of %s does not match %s', $dest_etag, $this->etag_if_match) );
       }
       else if ( isset($this->etag_none_match) && $this->etag_none_match != ''
                    && ($this->etag_none_match == $dest_etag || $this->etag_none_match == '*') ) {
@@ -1167,7 +1122,7 @@ EOSQL;
 
   }
 
-  
+
   /**
   * Is the user has the privileges to do what is requested.
   */
@@ -1271,7 +1226,7 @@ EOSQL;
   public static function kill_on_exit() {
     posix_kill( getmypid(), 28 );
   }
-  
+
   /**
   * Utility function we call when we have a simple status-based response to
   * return to the client.  Possibly
@@ -1293,7 +1248,7 @@ EOSQL;
       foreach( $lines AS $v ) {
         dbg_error_log( "LOG headers", "-->%s", $v );
       }
-      dbg_error_log( "LOG ", "******************** Response ********************" );
+      dbg_error_log( "LOG", "******************** Response ********************" );
       // Log the request in all it's gory detail.
       $lines = preg_split( '#[\r\n]+#', $message);
       foreach( $lines AS $v ) {
@@ -1301,20 +1256,22 @@ EOSQL;
       }
     }
 
+    $script_finish = microtime(true);
+    $script_time = $script_finish - $c->script_start_time;
+    $message_length = strlen($message);
     if ( $message != '' ) {
-      if ( !headers_sent() ) header( "Content-Length: ".strlen($message) );
+      if ( !headers_sent() ) header( "Content-Length: ".$message_length );
       echo $message;
     }
 
     if ( isset($c->dbg['caldav']) && $c->dbg['caldav'] ) {
-      if ( strlen($message) > 100 || strstr($message, "\n") ) {
-        $message = substr( preg_replace("#\s+#m", ' ', $message ), 0, 100) . (strlen($message) > 100 ? "..." : "");
+      if ( $message_length > 100 || strstr($message, "\n") ) {
+        $message = substr( preg_replace("#\s+#m", ' ', $message ), 0, 100) . ($message_length > 100 ? "..." : "");
       }
 
       dbg_error_log("caldav", "Status: %d, Message: %s, User: %d, Path: %s", $status, $message, $session->principal->user_no(), $this->path);
     }
     if ( isset($c->dbg['statistics']) && $c->dbg['statistics'] ) {
-      $script_time = microtime(true) - $c->script_start_time;
       $memory = '';
       if ( function_exists('memory_get_usage') ) {
         $memory = sprintf( ', Memory: %dk, Peak: %dk', memory_get_usage()/1024, memory_get_peak_usage(true)/1024);
@@ -1327,14 +1284,97 @@ EOSQL;
     }
     catch( Exception $ignored ) {}
 
+    if ( isset($c->metrics_style) && $c->metrics_style !== false ) {
+      $flush_time = microtime(true) - $script_finish;
+      $this->DoMetrics($status, $message_length, $script_time, $flush_time);
+    }
+
     if ( isset($c->exit_after_memory_exceeds) && function_exists('memory_get_peak_usage') && memory_get_peak_usage(true) > $c->exit_after_memory_exceeds ) { // 64M
       @dbg_error_log("statistics", "Peak memory use exceeds %d bytes (%d) - killing process %d", $c->exit_after_memory_exceeds, memory_get_peak_usage(true), getmypid());
       register_shutdown_function( 'CalDAVRequest::kill_on_exit' );
     }
-    
-    
+
     exit(0);
   }
 
+
+  /**
+  * Record the metrics related to this request.
+  *
+  * @param status The HTTP status code for this response
+  * @param response_size The size of the response (bytes).
+  * @param script_time The time taken to generate the response (pre-sending)
+  * @param flush_time The time taken to send the response (buffers flushed)
+  */
+  function DoMetrics($status, $response_size, $script_time, $flush_time) {
+    global $c;
+    static $ns = 'metrics';
+
+    $method = (empty($this->method) ? 'UNKNOWN' : $this->method);
+
+    // If they want 'both' or 'all' or something then that's what they will get
+    // If they don't want counters, they must want to use memcache!
+    if ( $c->metrics_style != 'counters' ) {
+      $cache = getCacheInstance();
+      if ( $cache->isActive() ) {
+
+        $base_key = $method.':';
+        $count_like_this = $cache->increment( $ns, $base_key.$status );
+        $cache->increment( $ns, $base_key.'size', $response_size );
+        $cache->increment( $ns, $base_key.'script_time', intval($script_time * 1000000) );
+        $cache->increment( $ns, $base_key.'flush_time', intval($flush_time * 1000000) );
+        $cache->increment( $ns, $base_key.'query_time', intval($c->total_query_time * 1000000) );
+
+        if ( $count_like_this == 1 ) {
+          // We need to maintain a set of details regarding the methods and statuses we have
+          // encountered, so we know what to retrieve.  Since this is the first one like
+          // this, we add it to the index.
+          try {
+            $index = unserialize($cache->get($ns, 'index'));
+          } catch (Exception $e) {
+            $index = array('methods' => array(), 'statuses' => array());
+          }
+          $index['methods'][$method] = 1;
+          $index['statuses'][$status] = 1;
+          $cache->set($ns, 'index', serialize($index), 0);
+        }
+      }
+      else {
+        error_log("Full statistics are only available with a working Memcache configuration");
+      }
+    }
+
+    // If they don't want memcache, they must want to use counters!
+    if ( $c->metrics_style != 'memcache' ) {
+      $qstring = "SELECT nextval('%s')";
+      switch( $method ) {
+        case 'OPTIONS':
+        case 'REPORT':
+        case 'PROPFIND':
+        case 'GET':
+        case 'PUT':
+        case 'HEAD':
+        case 'PROPPATCH':
+        case 'POST':
+        case 'MKCALENDAR':
+        case 'MKCOL':
+        case 'DELETE':
+        case 'MOVE':
+        case 'ACL':
+        case 'LOCK':
+        case 'UNLOCK':
+        case 'MKTICKET':
+        case 'DELTICKET':
+        case 'BIND':
+          $counter = strtolower($this->method);
+          break;
+        default:
+          $counter = 'unknown';
+          break;
+      }
+      $qry = new AwlQuery( "SELECT nextval('metrics_count_" . $counter . "')" );
+      $qry->Exec('always',__LINE__,__FILE__);
+    }
+  }
 }
 
