@@ -143,6 +143,27 @@ class DAVResource
   */
   private $tickets;
 
+  private $collection_type;
+  private $created;
+  private $displayname;
+  private $modified;
+  private $path_privs;
+  private $principal_id;
+  private $resource_id;
+  private $user_no;
+  private $access_tickets;
+  private $parent_container_type;
+  private $sync_token;
+
+  /**
+  * @var boolean A flag to indicate if the current collection is eligible for caching.
+  *
+  * This feels like a hack, but we store some metadata about a collection in
+  * $this which doesn't get cached, reconstructing that without the info from
+  * the database is fraught with pain.
+  */
+  private $_collection_is_cacheable;
+
   /**
   * Constructor
   * @param mixed $parameters If null, an empty Resourced is created.
@@ -174,7 +195,8 @@ class DAVResource
     $this->_is_binding       = false;
     $this->_is_external      = false;
     $this->_is_addressbook   = false;
-    $this->_is_proxy_resource = false;
+    $this->_is_proxy_resource       = false;
+    $this->_collection_is_cacheable = false;
 
     if ( isset($prefetched_collection) ) {
       $this->collection = $prefetched_collection;
@@ -379,6 +401,7 @@ class DAVResource
     if ( $qry->Exec('DAVResource') && $qry->rows() == 1 && ($row = $qry->Fetch()) ) {
       $this->collection = $row;
       $this->collection->exists = true;
+
       if ( $row->is_calendar == 't' )
         $this->collection->type = 'calendar';
       else if ( $row->is_addressbook == 't' )
@@ -387,6 +410,9 @@ class DAVResource
         $this->collection->type = 'schedule-'. $matches[3]. 'box';
       else
         $this->collection->type = 'collection';
+
+      # Vanilla collection, safe to cache.
+      $this->_collection_is_cacheable = true;
     }
     else if ( preg_match( '{^( ( / ([^/]+) / ) \.(in|out)/ ) [^/]*$}x', $this->dav_name, $matches ) ) {
       // The request is for a scheduling inbox or outbox (or something inside one) and we should auto-create it
@@ -411,16 +437,23 @@ EOSQL;
         $this->collection = $row;
         $this->collection->exists = true;
         $this->collection->type = $this->collection_type;
+
+        # Vanilla collection, safe to cache.
+        $this->_collection_is_cacheable = true;
       }
     }
     else if ( preg_match( '#^(/([^/]+)/calendar-proxy-(read|write))/?[^/]*$#', $this->dav_name, $matches ) ) {
       $this->collection->type = 'proxy';
       $this->_is_proxy_resource = true;
       $this->proxy_type = $matches[3];
+      $this->collection->proxy_type = $matches[3];
       $this->collection->dav_name = $this->dav_name;
       $this->collection->dav_displayname = sprintf( '%s proxy %s', $matches[2], $matches[3] );
       $this->collection->exists = true;
       $this->collection->parent_container = '/' . $matches[2] . '/';
+
+      # Proxy collection, fetch from cache can handle the _is_proxy_resource and proxy_type metadata.
+      $this->_collection_is_cacheable = true;
     }
     else if ( preg_match( '#^(/[^/]+)/?$#', $this->dav_name, $matches)
            || preg_match( '#^((/principals/[^/]+/)[^/]+)/?$#', $this->dav_name, $matches) ) {
@@ -428,6 +461,9 @@ EOSQL;
       $this->FetchPrincipal();
       $this->collection->is_principal = true;
       $this->collection->type = 'principal';
+
+      # We don't cache principals as a collection.
+      $this->_collection_is_cacheable = false;
    }
     else if ( $this->dav_name == '/' ) {
       $this->collection->dav_name = '/';
@@ -436,6 +472,9 @@ EOSQL;
       $this->collection->displayname = $c->system_name;
       $this->collection->default_privileges = (1 | 16 | 32);
       $this->collection->parent_container = '/';
+
+      # Vanilla collection, safe to cache.
+      $this->_collection_is_cacheable = true;
     }
     else {
       $sql = <<<EOSQL
@@ -474,7 +513,7 @@ EOSQL;
           $this->collection->type = 'schedule-'. $matches[3]. 'box';
         else
           $this->collection->type = 'collection';
-        if ( strlen($row->external_url) > 8 ) {
+        if ( isset($row->external_url) && strlen($row->external_url) > 8 ) {
           $this->_is_external = true;
           if ( $row->external_type == 'calendar' )
             $this->collection->type = 'calendar';
@@ -489,11 +528,18 @@ EOSQL;
           if ( !isset($this->tickets) ) $this->tickets = array();
           $this->tickets[] = new DAVTicket($row->access_ticket_id);
         }
+
+
+        # A lot of metadata is stored outside of the Collection, and the
+        # Collection is what we cache. Caching this type isn't safe as we
+        # can't restore all the metadata.
+        $this->_collection_is_cacheable = false;
       }
       else {
         dbg_error_log( 'DAVResource', 'No collection for path "%s".', $this->dav_name );
         $this->collection->exists = false;
         $this->collection->dav_name = preg_replace('{/[^/]*$}', '/', $this->dav_name);
+        $this->_collection_is_cacheable = false;
       }
     }
 
@@ -525,7 +571,7 @@ EOSQL;
     $this->collection = $cache->get( $cache_ns, $cache_key );
     if ( $this->collection === false ) {
       $this->ReadCollectionFromDatabase();
-      if ( $this->collection->type != 'principal' ) {
+      if ( $this->collection->type != 'principal' && $this->_collection_is_cacheable ) {
         $cache_ns = 'collection-'.$this->collection->dav_name;
         @dbg_error_log( 'Cache', ':FetchCollection: Setting cache ns "%s" key "%s". Type: %s', $cache_ns, $cache_key, $this->collection->type );
         $cache->set( $cache_ns, $cache_key, $this->collection );
@@ -540,6 +586,11 @@ EOSQL;
         $this->FetchPrincipal();
         $this->collection->is_principal = true;
         $this->collection->type = 'principal';
+      } else {
+        if ($this->collection->type == 'proxy') {
+          $this->_is_proxy_resource = true;
+          $this->proxy_type = $this->collection->proxy_type;
+        }
       }
       @dbg_error_log( 'DAVResource', ':FetchCollection: Read cached collection named "%s" of type "%s".', $this->collection->dav_name, $this->collection->type );
     }
@@ -686,8 +737,7 @@ EOQRY;
       }
     }
     xml_parser_free($xml_parser);
-    $position = 0;
-    $xmltree = BuildXMLTree( $xml_tags, $position);
+    $xmltree = BuildXMLTree( $xml_tags );
     return $xmltree->GetContent();
   }
 
