@@ -55,6 +55,14 @@ class HTTPAuthSession {
   * @var groups array
   */
   public $groups;
+
+  /* These fields were being added dynmically. Set to private mostly, I had to
+   * set these as public for tests to pass: principal
+   */
+  private $dav_name;
+  private $logged_in;
+  public $principal;
+  private $roles;
   /**#@-*/
 
   /**
@@ -377,10 +385,6 @@ class HTTPAuthSession {
       $principal = new Principal('username',$principal->username);
     }
 
-    // Assign each field in the selected record to the object
-    foreach( $principal AS $k => $v ) {
-      $this->{$k} = $v;
-    }
     if ( !get_class($principal) == 'Principal' ) {
       throw new Exception('HTTPAuthSession::AssignSessionDetails could not find a Principal object');
     }
