@@ -234,7 +234,10 @@ class Principal {
       }
       $this->assignRowValues($row);
       $this->url = ConstructURL( $this->dav_name, true );
-      $row = $cache->set($this->cacheNs, $this->cacheKey, $row, 864000 );
+
+      if (isset($this->cacheNs))
+        $cache->set($this->cacheNs, $this->cacheKey, $row, 864000 );
+
       return $this;
     }
 
