@@ -125,14 +125,15 @@ class WritableCollection extends DAVResource {
     }
 
     $dtstart = $first->GetPValue('DTSTART');
-    $calitem_params[':dtstart'] = $dtstart;
+    $calitem_params[':dtstart_orig'] = $dtstart;
     if ( (!isset($dtstart) || $dtstart == '') && $first->GetPValue('DUE') != '' ) {
       $dtstart = $first->GetPValue('DUE');
     }
 
     $dtend = $first->GetPValue('DTEND');
+    $calitem_params[':dtend_orig'] = $dtend;
     if ( isset($dtend) && $dtend != '' ) {
-      dbg_error_log( 'PUT', ' DTEND: "%s", DTSTART: "%s", DURATION: "%s"', $dtend, $dtstart, $first->GetPValue('DURATION') );
+      dbg_error_log( 'PUT', ' DTEND: "%s", DTSTART: "%s", DURATION: "%s"', $dtend, $first->GetPValue('DTSTART'), $first->GetPValue('DURATION') );
       $calitem_params[':dtend'] = $dtend;
       $dtend = ':dtend';
     }
@@ -180,11 +181,7 @@ class WritableCollection extends DAVResource {
     $calitem_params[':dtstamp'] = $dtstamp;
 
     $class = $first->GetPValue('CLASS');
-    /*
-     * It seems that some calendar clients don't set a class...
-     * RFC2445, 4.8.1.3: Default is PUBLIC
-     */
-    if ( $this->IsPublicOnly() || !isset($class) || $class == '' ) {
+    if ( $this->IsPublicOnly() && isset($class)) {
       $class = 'PUBLIC';
     }
     $calitem_params[':class'] = $class;
@@ -207,46 +204,50 @@ class WritableCollection extends DAVResource {
     if ( $created == '00001231T000000Z' ) $created = '20001231T000000Z';
     $calitem_params[':created'] = $created;
 
-    $calitem_params[':tzid'] = $tzid;
-    $calitem_params[':uid'] = $uid;
-    $calitem_params[':summary'] = $first->GetPValue('SUMMARY');
-    $calitem_params[':location'] = $first->GetPValue('LOCATION');
-    $calitem_params[':transp'] = $first->GetPValue('TRANSP');
-    $calitem_params[':description'] = $first->GetPValue('DESCRIPTION');
+    $calitem_params[':tzid']  = $tzid;
+    $calitem_params[':uid']   = $uid;
+    $calitem_params[':url']   = $first->GetPValue('URL');
+    $calitem_params[':due']   = $first->GetPValue('DUE');
     $calitem_params[':rrule'] = $first->GetPValue('RRULE');
-    $calitem_params[':url'] = $first->GetPValue('URL');
+    $calitem_params[':dtstart']  = $first->GetPValue('DTSTART');
+    $calitem_params[':summary']  = $first->GetPValue('SUMMARY');
+    $calitem_params[':location'] = $first->GetPValue('LOCATION');
+    $calitem_params[':transp']   = $first->GetPValue('TRANSP');
+    $calitem_params[':status']   = $first->GetPValue('STATUS');
     $calitem_params[':priority'] = $first->GetPValue('PRIORITY');
-    $calitem_params[':due'] = $first->GetPValue('DUE');
+    $calitem_params[':description']      = $first->GetPValue('DESCRIPTION');
     $calitem_params[':percent_complete'] = $first->GetPValue('PERCENT-COMPLETE');
-    $calitem_params[':status'] = $first->GetPValue('STATUS');
 
     $range = getVCalendarRange($vcal, $this->timezone_name());
-    $calitem_params[':first_instance_start'] = isset($range->from) ? $range->from->UTC() : null;
-    $calitem_params[':last_instance_end'] = isset($range->until) ? $range->until->UTC() : null;
+    $calitem_params[':first_instance_start'] = isset($range->from)  ? $range->from->UTC()  : null;
+    $calitem_params[':last_instance_end']    = isset($range->until) ? $range->until->UTC() : null;
 
     if ( $create_resource ) {
       $sql = <<<EOSQL
 INSERT INTO calendar_item (user_no, dav_name, dav_id, dav_etag, uid, dtstamp,
-                dtstart, dtend, summary, location, class, transp,
-                description, rrule, tz_id, last_modified, url, priority,
-                created, due, percent_complete, status, collection_id,
-                first_instance_start, last_instance_end )
-   VALUES ( :user_no, :dav_name, currval('dav_id_seq'), :etag, :uid, :dtstamp,
-                :dtstart, $dtend, :summary, :location, :class, :transp,
-                :description, :rrule, :tzid, :modified, :url, :priority,
-                :created, :due, :percent_complete, :status, $collection_id,
-                :first_instance_start, :last_instance_end)
+  dtstart, dtstart_orig, dtend, dtend_orig, summary, location, class, transp,
+  description, rrule, tz_id, last_modified, url, priority, created, due,
+  percent_complete, status, collection_id, first_instance_start,
+  last_instance_end )
+VALUES ( :user_no, :dav_name, currval('dav_id_seq'), :etag, :uid, :dtstamp,
+  :dtstart, :dtstart_orig, $dtend, :dtend_orig, :summary, :location, :class,
+  :transp, :description, :rrule, :tzid, :modified, :url, :priority,
+  :created, :due, :percent_complete, :status, $collection_id,
+  :first_instance_start, :last_instance_end)
 EOSQL;
       $sync_change = 201;
     }
     else {
       $sql = <<<EOSQL
 UPDATE calendar_item SET dav_etag=:etag, uid=:uid, dtstamp=:dtstamp,
-                dtstart=:dtstart, dtend=$dtend, summary=:summary, location=:location, class=:class, transp=:transp,
-                description=:description, rrule=:rrule, tz_id=:tzid, last_modified=:modified, url=:url, priority=:priority,
-                created=:created, due=:due, percent_complete=:percent_complete, status=:status,
-                first_instance_start=:first_instance_start, last_instance_end=:last_instance_end
-       WHERE user_no=:user_no AND dav_name=:dav_name
+  dtstart=:dtstart, dtstart_orig=:dtstart_orig, dtend=$dtend,
+  dtend_orig=:dtend_orig, summary=:summary, location=:location, class=:class,
+  transp=:transp, description=:description, rrule=:rrule, tz_id=:tzid,
+  last_modified=:modified, url=:url, priority=:priority, created=:created,
+  due=:due, percent_complete=:percent_complete, status=:status,
+  first_instance_start=:first_instance_start,
+  last_instance_end=:last_instance_end
+WHERE user_no=:user_no AND dav_name=:dav_name
 EOSQL;
       $sync_change = 200;
     }
@@ -326,10 +327,11 @@ EOSQL;
 
     $components = $vcal->GetComponents();
 
-    $qry->SetSql('INSERT INTO calendar_alarm ( dav_id, action, trigger, summary, description, component, next_trigger )
-            VALUES( '.$dav_id.', :action, :trigger, :summary, :description, :component,
+    $qry->SetSql('INSERT INTO calendar_alarm ( dav_id, count, action, trigger, summary, description, component, next_trigger )
+            VALUES( '.$dav_id.', :count, :action, :trigger, :summary, :description, :component,
                                         :related::timestamp with time zone + :related_trigger::interval )' );
     $qry->Prepare();
+    $count = 0;
     foreach( $components AS $component ) {
       if ( $component->GetType() == 'VTIMEZONE' ) continue;
       $alarms = $component->GetComponents('VALARM');
@@ -370,6 +372,7 @@ EOSQL;
         $qry->Bind(':component', $v->Render());
         $qry->Bind(':related', $related );
         $qry->Bind(':related_trigger', $related_trigger );
+        $qry->Bind(':count', $count++ );
         $qry->Exec('PUT',__LINE__,__FILE__);
       }
     }
