@@ -88,8 +88,7 @@ $resource = new DAVResource( $request->path );
    would allow unauthenticated access to resources.
 */
 
-$position = 0;
-$xmltree = BuildXMLTree( $request->xml_tags, $position);
+$xmltree = BuildXMLTree( $request->xml_tags );
 $aces = $xmltree->GetPath("/DAV::acl/*");
 
 $grantor = new DAVResource($request->path);
@@ -174,7 +173,8 @@ function process_ace( $grantor, $by_principal, $by_collection, $ace ) {
 
     case 'DAV::authenticated':
       $principal_type = 'authenticated';
-      if ( bindec($grantor->GetProperty('default_privileges')) == $privileges ) break; // There is no change, so skip it
+      $grant_default_privs = $grantor->GetProperty('default_privileges');
+      if ( isset($grant_default_privs) && bindec($grant_default_privs) == $privileges ) break; // There is no change, so skip it
       $sqlparms = array( ':privileges' => $privileges );
       if ( isset($by_collection) ) {
         $sql = 'UPDATE collection SET default_privileges=:privileges::INT::BIT(24) WHERE collection_id=:by_collection';
