@@ -55,10 +55,14 @@ function export_iCalendar( DAVResource $dav_resource ) {
     $params = array( ':collection_id' => $dav_resource->resource_id() );
     $distinct = '';
   }
-  $sql = 'SELECT '.$distinct.' caldav_data, class, caldav_type, calendar_item.user_no, logged_user ';
+  $sql = 'SELECT caldav_data, class, caldav_type, user_no, logged_user ';
+  $sql .= 'FROM (';
+  $sql .= 'SELECT '.$distinct.' caldav_data, class, caldav_type, calendar_item.user_no, logged_user, calendar_item.dav_name, calendar_item.dtstart, calendar_item.dtend ';
   $sql .= 'FROM collection INNER JOIN caldav_data USING(collection_id) ';
   $sql .= 'INNER JOIN calendar_item USING ( dav_id ) WHERE '.$where;
   if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= ' ORDER BY calendar_item.uid, calendar_item.dav_id';
+  $sql .= ') AS s1';
+  if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= ' ORDER BY user_no, dtstart, dtend';
 
   $qry = new AwlQuery( $sql, $params );
   if ( !$qry->Exec("GET",__LINE__,__FILE__) ) {
