@@ -17,12 +17,15 @@ $dav_resource->NeedPrivilege( array('urn:ietf:params:xml:ns:caldav:read-free-bus
 if ( $dav_resource->IsExternal() ) {
   require_once("external-fetch.php");
   update_external ( $dav_resource );
-}      
+}
 
 if ( ! $dav_resource->Exists() ) {
   $request->DoResponse( 404, translate("Resource Not Found.") );
 }
 
+if ( $request->path == '/' ) {
+  $request->DoResponse( 404, translate("Resource Not Found.") );
+}
 
 if ( $dav_resource->IsCollection() ) {
   $response = export_iCalendar($dav_resource);
@@ -72,7 +75,7 @@ switch( $resource->caldav_type ) {
   case 'VJOURNAL':
   case 'VEVENT':
   case 'VTODO':
-    $contenttype = 'text/calendar';
+    $contenttype = 'text/calendar; component=' . strtolower($resource->caldav_type);
     break;
 
   case 'VCARD':
