@@ -23,8 +23,7 @@ if ( !isset($request->xml_tags) ) {
   $property_list = array('DAV::allprop');
 }
 else {
-  $position = 0;
-  $xmltree = BuildXMLTree( $request->xml_tags, $position);
+  $xmltree = BuildXMLTree( $request->xml_tags );
   if ( !is_object($xmltree) ) {
     $request->DoResponse( 403, translate("Request body is not valid XML data!") );
   }
