@@ -210,7 +210,8 @@ function handle_schedule_request( $ical ) {
       dbg_error_log( "PUT", "not delivering to owner" );
       continue;
     }
-    if ( $attendee->GetParameterValue ( 'PARTSTAT' ) != 'NEEDS-ACTION' || preg_match ( '/^[35]\.[3-9]/',  $attendee->GetParameterValue ( 'SCHEDULE-STATUS' ) ) ) {
+    $schedule_status = $attendee->GetParameterValue ( 'SCHEDULE-STATUS' );
+    if ( $attendee->GetParameterValue ( 'PARTSTAT' ) != 'NEEDS-ACTION' || (isset($schedule_status) && preg_match ( '/^[35]\.[3-9]/',  $schedule_status ) ) ) {
       dbg_error_log( "PUT", "attendee %s does not need action", $attendee_email );
       continue;
     }
@@ -640,7 +641,7 @@ function do_scheduling_requests( vCalendar $resource, $create, $old_data = null
         $response = '5.2';  // No scheduling support for user
       }
       else {
-        if ($attendee_is_new || !$row) {
+        if ($attendee_is_new || !isset($row)) {
           $this_schedule_request = clone($schedule_request);
           $this_resource = clone($resource);
         } else {
@@ -1036,19 +1037,25 @@ UPDATE caldav_data SET user_no=:user_no, caldav_data=:dav_data, dav_etag=:etag,
 EOSQL;
 
   $calitem_insert = <<<EOSQL
-INSERT INTO calendar_item (user_no, dav_name, dav_id, dav_etag, uid, dtstamp, dtstart, dtend, summary, location, class, transp,
-                    description, rrule, tz_id, last_modified, url, priority, created, due, percent_complete, status, collection_id )
-    VALUES ( :user_no, :dav_name, currval('dav_id_seq'), :etag, :uid, :dtstamp, :dtstart, ##dtend##, :summary, :location, :class, :transp,
-                :description, :rrule, :tzid, :modified, :url, :priority, :created, :due, :percent_complete, :status, :collection_id)
+INSERT INTO calendar_item (user_no, dav_name, dav_id, dav_etag, uid, dtstamp,
+  dtstart, dtstart_orig, dtend, dtend_orig, summary, location, class, transp,
+  description, rrule, tz_id, last_modified, url, priority, created, due,
+  percent_complete, status, collection_id )
+VALUES ( :user_no, :dav_name, currval('dav_id_seq'), :etag, :uid, :dtstamp,
+  :dtstart, :dtstart_orig, ##dtend##, :dtend_orig, :summary, :location,
+  :class, :transp, :description, :rrule, :tzid, :modified, :url, :priority,
+  :created, :due, :percent_complete, :status, :collection_id)
 EOSQL;
 
   $calitem_update = <<<EOSQL
-UPDATE calendar_item SET user_no=:user_no, dav_etag=:etag, uid=:uid, dtstamp=:dtstamp,
-                dtstart=:dtstart, dtend=##dtend##, summary=:summary, location=:location,
-                class=:class, transp=:transp, description=:description, rrule=:rrule,
-                tz_id=:tzid, last_modified=:modified, url=:url, priority=:priority,
-                due=:due, percent_complete=:percent_complete, status=:status
-       WHERE collection_id=:collection_id AND dav_name=:dav_name
+UPDATE calendar_item
+SET user_no=:user_no, dav_etag=:etag, uid=:uid, dtstamp=:dtstamp, dtstart=:dtstart,
+  dtstart_orig=:dtstart_orig, dtend=##dtend##, summary=:summary,
+  location=:location, class=:class, transp=:transp, description=:description,
+  rrule=:rrule, tz_id=:tzid, last_modified=:modified, url=:url,
+  priority=:priority, due=:due, percent_complete=:percent_complete,
+  status=:status
+WHERE collection_id=:collection_id AND dav_name=:dav_name
 EOSQL;
 
   $last_olson = '';
@@ -1079,7 +1086,9 @@ EOSQL;
     $dav_data_params[':session_user'] = $session->user_no;
 
     $dtstart = $first->GetPValue('DTSTART');
+    $calitem_params[':dtstart_orig'] = $dtstart;
     $calitem_params[':dtstart'] = $dtstart;
+
     if ( (!isset($dtstart) || $dtstart == '') && $first->GetPValue('DUE') != '' ) {
       $dtstart = $first->GetPValue('DUE');
       if ( isset($after) ) $dtstart_date = new RepeatRuleDateTime($first->GetProperty('DUE'));
@@ -1120,6 +1129,8 @@ EOSQL;
     }
 
     $dtend = $first->GetPValue('DTEND');
+    $calitem_params[':dtend_orig'] = $dtend;
+
     if ( isset($dtend) && $dtend != '' ) {
       dbg_error_log( 'PUT', ' DTEND: "%s", DTSTART: "%s", DURATION: "%s"', $dtend, $dtstart, $first->GetPValue('DURATION') );
       $calitem_params[':dtend'] = $dtend;
@@ -1169,9 +1180,8 @@ EOSQL;
     if ( empty($dtstamp) ) $dtstamp = $last_modified;
     $calitem_params[':dtstamp'] = $dtstamp;
 
-    /** RFC2445, 4.8.1.3: Default is PUBLIC, or also if overridden by the collection settings */
-    $class = ($collection->public_events_only == 't' ? 'PUBLIC' : $first->GetPValue('CLASS') );
-    if ( !isset($class) || $class == '' ) $class = 'PUBLIC';
+    /** Allow overridding class by the collection settings */
+    $class = ($collection->public_events_only == 't' && isset($class) ? 'PUBLIC' : $first->GetPValue('CLASS') );
     $calitem_params[':class'] = $class;
 
 
@@ -1203,17 +1213,21 @@ EOSQL;
     }
 
     $sql = str_replace( '##dtend##', $dtend, ($inserting ? $calitem_insert : $calitem_update) );
+    $calitem_params[':uid']  = $first->GetPValue('UID');
+    $calitem_params[':url']  = $first->GetPValue('URL');
+    $calitem_params[':due']  = $first->GetPValue('DUE');
     $calitem_params[':tzid'] = $tzid;
-    $calitem_params[':uid'] = $first->GetPValue('UID');
-    $calitem_params[':summary'] = $first->GetPValue('SUMMARY');
+    $calitem_params[':transp']   = $first->GetPValue('TRANSP');
+    $calitem_params[':status']   = $first->GetPValue('STATUS');
+    $calitem_params[':summary']  = $first->GetPValue('SUMMARY');
     $calitem_params[':location'] = $first->GetPValue('LOCATION');
-    $calitem_params[':transp'] = $first->GetPValue('TRANSP');
-    $calitem_params[':description'] = $first->GetPValue('DESCRIPTION');
-    $calitem_params[':url'] = $first->GetPValue('URL');
     $calitem_params[':priority'] = $first->GetPValue('PRIORITY');
-    $calitem_params[':due'] = $first->GetPValue('DUE');
+    $calitem_params[':description']  = $first->GetPValue('DESCRIPTION');
     $calitem_params[':percent_complete'] = $first->GetPValue('PERCENT-COMPLETE');
-    $calitem_params[':status'] = $first->GetPValue('STATUS');
+
+    // Intentionally not populating first_instance_start and last_instance_end
+    // here, As they may depend on the default tzid of the collection, which as
+    // I understand it hasn't yet been set
 
     if ( $inserting ) {
       $created = $first->GetPValue('CREATED');
@@ -1265,10 +1279,11 @@ function write_alarms( $dav_id, vComponent $ical ) {
   $alarms = $ical->GetComponents('VALARM');
   if ( count($alarms) < 1 ) return;
 
-  $qry->SetSql('INSERT INTO calendar_alarm ( dav_id, action, trigger, summary, description, component, next_trigger )
-          VALUES( '.$dav_id.', :action, :trigger, :summary, :description, :component,
+  $qry->SetSql('INSERT INTO calendar_alarm ( dav_id, count, action, trigger, summary, description, component, next_trigger )
+          VALUES( '.$dav_id.', :count, :action, :trigger, :summary, :description, :component,
                                       :related::timestamp with time zone + :related_trigger::interval )' );
   $qry->Prepare();
+  $count = 0;
   foreach( $alarms AS $v ) {
     $trigger = array_merge($v->GetProperties('TRIGGER'));
     if ( $trigger == null ) continue; // Bogus data.
@@ -1309,6 +1324,7 @@ function write_alarms( $dav_id, vComponent $ical ) {
     $qry->Bind(':component', $v->Render());
     $qry->Bind(':related', $related_date->UTC() );
     $qry->Bind(':related_trigger', $related_trigger );
+    $qry->Bind(':count', $count++ );
     $qry->Exec('PUT',__LINE__,__FILE__);
   }
 }
@@ -1426,11 +1442,11 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
     if (strcmp($dtstart, $olddtstart)) $modified = true;
     $dtend = $first->GetPValue('DTEND');
     $olddtend = $oldfirst->GetPValue('DTEND');
-    if (strcmp($dtend, $olddtend)) $modified = true;
+    if (isset($dtend) && isset($olddtend) && strcmp($dtend, $olddtend)) $modified = true;
     $duration = $first->GetPValue('DURATION');
     $oldduration = $oldfirst->GetPValue('DURATION');
     $organizer = $vcal->GetOrganizer();
-    if (strcmp($duration, $oldduration)) $modified = true;
+    if (isset($duration) && isset($oldduration) && strcmp($duration, $oldduration)) $modified = true;
     if (($modified == true) && !($organizer === false || empty($organizer))) {
       dbg_error_log( 'PUT', "event time attributes have been modified, reset all attendees' PARTSTAT to NEEDS-ACTION.");
       $organizer_email = preg_replace( '/^mailto:/i', '', $organizer->Value() );
@@ -1463,12 +1479,16 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   if ( $first->GetType() == 'VTODO' ) $due = $first->GetPValue('DUE');
   $calitem_params[':due'] = $due;
   $dtstart = $first->GetPValue('DTSTART');
+  $calitem_params[':dtstart_orig'] = $dtstart;
+
   if ( empty($dtstart) ) $dtstart = $due;
-  if (preg_match("/^1[0-8][0-9][0-9][01][0-9][0-3][0-9]$/", $dtstart))
+  if (isset($dtstart) && preg_match("/^1[0-8][0-9][0-9][01][0-9][0-3][0-9]$/", $dtstart))
      $dtstart = $dtstart . "T000000Z";
   $calitem_params[':dtstart'] = $dtstart;
 
   $dtend = $first->GetPValue('DTEND');
+  $calitem_params[':dtend_orig'] = $dtend;
+
   if ( isset($dtend) && $dtend != '' ) {
     dbg_error_log( 'PUT', ' DTEND: "%s", DTSTART: "%s", DURATION: "%s"', $dtend, $dtstart, $first->GetPValue('DURATION') );
     if (preg_match("/^1[0-8][0-9][0-9][01][0-9][0-3][0-9]$/", $dtend))
@@ -1527,18 +1547,10 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   $class = $first->GetPValue('CLASS');
   /* Check and see if we should over ride the class. */
   /** @todo is there some way we can move this out of this function? Or at least get rid of the need for the SQL query here. */
-  if ( public_events_only($user_no, $path) ) {
+  if ( public_events_only($user_no, $path) && isset($class) ) {
     $class = 'PUBLIC';
   }
 
-  /*
-   * It seems that some calendar clients don't set a class...
-   * RFC2445, 4.8.1.3:
-   * Default is PUBLIC
-   */
-  if ( !isset($class) || $class == '' ) {
-    $class = 'PUBLIC';
-  }
   $calitem_params[':class'] = $class;
 
   /** Calculate what timezone to set, first, if possible */
@@ -1596,7 +1608,11 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   $calitem_params[':percent_complete'] = $first->GetPValue('PERCENT-COMPLETE');
   $calitem_params[':status'] = $first->GetPValue('STATUS');
 
-  // force re-render the object (to get the same representation for all attendiees)
+  $range = getVCalendarRange($vcal, $resource->timezone_name());
+  $calitem_params[':first_instance_start'] = isset($range->from) ? $range->from->UTC() : null;
+  $calitem_params[':last_instance_end'] = isset($range->until) ? $range->until->UTC() : null;
+
+  // force re-render the object (to get the same representation for all attendees)
   $vcal->Render(null, true);
 
   if ( !$collection->IsSchedulingCollection() ) {
@@ -1630,13 +1646,15 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   if ( $put_action_type == 'INSERT' ) {
     $sql = <<<EOSQL
 INSERT INTO calendar_item (user_no, dav_name, dav_id, dav_etag, uid, dtstamp,
-                dtstart, dtend, summary, location, class, transp,
-                description, rrule, tz_id, last_modified, url, priority,
-                created, due, percent_complete, status, collection_id )
-   VALUES ( :user_no, :dav_name, :dav_id, :etag, :uid, :dtstamp,
-                :dtstart, $dtend, :summary, :location, :class, :transp,
-                :description, :rrule, :tzid, :modified, :url, :priority,
-                :created, :due, :percent_complete, :status, :collection_id )
+  dtstart, dtstart_orig, dtend, dtend_orig, summary, location, class, transp,
+  description, rrule, tz_id, last_modified, url, priority, created, due,
+  percent_complete, status, collection_id, first_instance_start,
+  last_instance_end )
+VALUES ( :user_no, :dav_name, :dav_id, :etag, :uid, :dtstamp, :dtstart,
+  :dtstart_orig, $dtend, :dtend_orig, :summary, :location, :class, :transp,
+  :description, :rrule, :tzid, :modified, :url, :priority, :created, :due,
+  :percent_complete, :status, :collection_id, :first_instance_start,
+  :last_instance_end)
 EOSQL;
     $sync_change = 201;
     $calitem_params[':collection_id'] = $collection_id;
@@ -1647,11 +1665,14 @@ EOSQL;
   else {
     $sql = <<<EOSQL
 UPDATE calendar_item SET dav_etag=:etag, uid=:uid, dtstamp=:dtstamp,
-                dtstart=:dtstart, dtend=$dtend, summary=:summary, location=:location,
-                class=:class, transp=:transp, description=:description, rrule=:rrule,
-                tz_id=:tzid, last_modified=:modified, url=:url, priority=:priority,
-                due=:due, percent_complete=:percent_complete, status=:status
-       WHERE dav_id=:dav_id
+  dtstart=:dtstart, dtstart_orig=:dtstart_orig, dtend=$dtend,
+  dtend_orig=:dtend_orig, summary=:summary, location=:location, class=:class,
+  transp=:transp, description=:description, rrule=:rrule, tz_id=:tzid,
+  last_modified=:modified, url=:url, priority=:priority, due=:due,
+  percent_complete=:percent_complete, status=:status,
+  first_instance_start=:first_instance_start,
+  last_instance_end=:last_instance_end
+WHERE dav_id=:dav_id
 EOSQL;
     $sync_change = 200;
   }
@@ -1672,7 +1693,8 @@ EOSQL;
     rollback_on_error( $caldav_context, $user_no, $path);
     return false;
   }
-  $qry->QDo("SELECT write_sync_change( $collection_id, $sync_change, :dav_name)", array(':dav_name' => $path ) );
+  $qry->QDo("SELECT write_sync_change( :collection_id, $sync_change, :dav_name)",
+     array(':collection_id' => $collection_id, ':dav_name' => $path ) );
   $qry->Commit();
 
   if ( function_exists('post_commit_action') ) {
