@@ -68,7 +68,6 @@ function calquery_apply_filter( $filters, $item ) {
 
 /**
  * Process a filter fragment returning an SQL fragment
- * Changed by GitHub user moosemark 2013-11-29 to allow multiple text-matches - SQL parameter now has numeric count appended.
  */
 $need_post_filter = false;
 $range_filter = null;
@@ -77,6 +76,7 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
   global $need_post_filter, $range_filter, $target_collection, $parameter_match_num;
   $sql = "";
   $params = array();
+  global $expand_range_start, $expand_range_end, $expand_as_floating;
   if ( !is_array($filter) ) {
     dbg_error_log( "calquery", "Filter is of type '%s', but should be an array of XML Tags.", gettype($filter) );
   }
@@ -86,35 +86,89 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
     dbg_error_log("calquery", "Processing $tag into SQL - %d, '%s', %d\n", count($components), $property, isset($parameter) );
 
     $not_defined = "";
+    $not_defined_log = "";
     switch( $tag ) {
       case 'urn:ietf:params:xml:ns:caldav:is-not-defined':
-        $not_defined = "not-"; // then fall through to IS-DEFINED case
+        $not_defined = "not ";     // for SQL
+        $not_defined_log = "not-"; // for logging
+        // then fall through to IS-DEFINED case
       case 'urn:ietf:params:xml:ns:caldav:is-defined':
         if ( isset( $parameter ) ) {
           $need_post_filter = true;
-          dbg_error_log("calquery", "Could not handle 'is-%sdefined' on property %s, parameter %s in SQL", $not_defined, $property, $parameter );
+          dbg_error_log("calquery", "Could not handle 'is-%sdefined' on property %s, parameter %s in SQL", $not_defined_log, $property, $parameter );
           return false;  // Not handled in SQL
         }
         if ( isset( $property ) ) {
-          switch( $property ) {
+          # $property may include the table name, remove that for the switch.
+          if (strpos($property, '.')) {
+            $base_property = explode('.', $property)[1];
+          } else {
+            $base_property = $property;
+          }
+
+          // For some tables we want to check that the user actually sent us,
+          // and that may be in a column with a '_orig' suffix.
+          $property_suffix = '';
+
+          switch( $base_property ) {
+            case 'dtend':
+            case 'dtstart':
+              $property_suffix = '_orig';
+
             case 'created':
             case 'completed':  /** @todo when it can be handled in the SQL - see around line 200 below */
             case 'dtend':
             case 'dtstamp':
             case 'dtstart':
+            case 'due':
               if ( ! $target_collection->IsSchedulingCollection() ) {
-                $property_defined_match = "IS NOT NULL";
+                if ($not_defined == "not ") {
+                  // "not IS NOT NULL" is a syntax error in SQL.
+                  $property_defined_match = "IS NULL";
+                  $not_defined = '';
+                } else {
+                  $property_defined_match = "IS NOT NULL";
+                }
               }
               break;
 
             case 'priority':
-              $property_defined_match = "IS NOT NULL";
+            case 'status':
+            case 'class':
+              if ($not_defined == "not ") {
+                // "not IS NOT NULL" is a syntax error in SQL.
+                $property_defined_match = "IS NULL";
+                $not_defined = '';
+              } else {
+                $property_defined_match = "IS NOT NULL";
+              }
+              break;
+
+            // These may be null or have the empty string, so we need a more complicated query.
+            case 'location':
+            case 'summary':
+            case 'url':
+              if ($not_defined == "not ") {
+                // "not IS NOT NULL" is a syntax error in SQL.
+                $property_defined_match = "IS NULL";
+                $condition = 'OR';
+              } else {
+                $property_defined_match = "IS NOT NULL";
+                $condition = 'AND';
+              }
+
+
+              $sql .= sprintf(" AND (%s %s %s %s %sLIKE '_%%') ", $property, $property_defined_match, $condition, $property, $not_defined);
+              $property_defined_match = null;
               break;
 
             default:
               $property_defined_match = "LIKE '_%'";  // i.e. contains a single character or more
           }
-          $sql .= sprintf( "AND %s %s%s ", $property, $not_defined, $property_defined_match );
+
+          if (isset($property_defined_match)) {
+            $sql .= sprintf( "AND %s%s %s%s ", $property, $property_suffix, $not_defined, $property_defined_match );
+          }
         }
         break;
 
@@ -129,12 +183,48 @@ function SqlFilterFragment( $filter, $components, $property = null, $parameter =
         $start = $v->GetAttribute("start");
         $finish = $v->GetAttribute("end");
 
-        if ( isset($start) )
+        if ( isset($start) ) {
           $params[':time_range_start'] = $start;
+          $time_range_start = new RepeatRuleDateTime($start);
+        } else {
+          # RFC 4791 Section 9.9 says that if start isn't set, then -infinity
+          # should be used. We can't specify that in PHP, and
+          # expand_event_instances has '-6 weeks' as the default.
+          # That's a bit short, go for one year prior to end, if defined,
+          # otherwise 1 year prior to now now.
+          if ( isset($finish) ) {
+            $time_range_start = new RepeatRuleDateTime($finish);
+          } else {
+            $time_range_start = new RepeatRuleDateTime;
+          }
+          $time_range_start->modify('-365 days');
 
-        if ( isset($finish) )
+          $params[':time_range_start'] = $time_range_start->UTC();
+        }
+
+        if (! isset($expand_range_start) || $time_range_start > $expand_range_start ) {
+          # We overload the expand_range_start, and we may need to make it more restrictive.
+          $expand_range_start = $time_range_start;
+        }
+
+        if ( isset($finish) ) {
           $params[':time_range_end'] = $finish;
+          $time_range_end = new RepeatRuleDateTime($finish);
+        } else {
+          # RFC 4791 Section 9.9 says that if end isn't set, then +infinity
+          # should be used. We can't specify that in PHP, and
+          # expand_event_instances has '+ 6 weeks' as the default. That's a
+          # bit short, go for two years from the start.
+          $time_range_end = clone($time_range_start);
+          $time_range_end->modify('+730 days');
 
+          $params[':time_range_end'] = $time_range_end->UTC();
+        }
+
+        if (! isset($expand_range_end) || $time_range_end < $expand_range_end ) {
+          # We overload the expand_range_end, and we may need to make it more restrictive.
+          $expand_range_end = $time_range_end;
+        }
 
         $legacy_start_cond = "($start_column IS NULL AND $finish_column > :time_range_start) OR $start_column > :time_range_start";
         $legacy_end_cond = "$finish_column < :time_range_end";
@@ -171,6 +261,9 @@ OR (first_instance_start IS NOT NULL AND ($new_cond))
         $search = $v->GetContent();
         $negate = $v->GetAttribute("negate-condition");
         $collation = $v->GetAttribute("collation");
+        if (! isset($collation)) {
+            $collation = '';
+        }
         switch( strtolower($collation) ) {
           case 'i;octet':
             $comparison = 'LIKE';
@@ -356,7 +449,7 @@ if ( $qry->Exec("calquery",__LINE__,__FILE__) && $qry->rows() > 0 ) {
         if ( $bound_from != $target_collection->dav_name() ) {
           $dav_object->dav_name = str_replace( $bound_from, $target_collection->dav_name(), $dav_object->dav_name);
         }
-        if ( $need_expansion ) {
+        if ( $need_expansion || isset($range_filter) ) {
           $vResource = new vComponent($dav_object->caldav_data);
           $expanded = getVCalendarRange($vResource, $dav_object->collection_tzid);
           if ( !$expanded->overlaps($range_filter) ) continue;
@@ -364,7 +457,13 @@ if ( $qry->Exec("calquery",__LINE__,__FILE__) && $qry->rows() > 0 ) {
           $expanded = expand_event_instances($vResource, $expand_range_start, $expand_range_end, $expand_as_floating , $dav_object->collection_tzid);
 
           if ( $expanded->ComponentCount() == 0 ) continue;
-          if ( $need_expansion ) $dav_object->caldav_data = $expanded->Render();
+
+          # We only keep the expanded instaneces if need_expansion is set,
+          # RFC4791 Section 7.4. Otherwise we should return the original
+          # caldav_data.
+          if ( $need_expansion ) {
+            $dav_object->caldav_data = $expanded->Render();
+          }
         }
         else if ( isset($range_filter) ) {
           $vResource = new vComponent($dav_object->caldav_data);
