@@ -33,7 +33,7 @@ if ( $sync_level == DEPTH_INFINITY ) {
 $sync_tokens = $xmltree->GetPath('/DAV::sync-collection/DAV::sync-token');
 if ( isset($sync_tokens[0]) ) $sync_token = $sync_tokens[0]->GetContent();
 if ( !isset($sync_token) ) $sync_token = 0;
-$sync_token = intval(str_ireplace('data:,', '', $sync_token ));
+$sync_token = intval(str_replace('data:,', '', strtolower($sync_token) ));
 dbg_error_log( 'sync', " sync-token: %s", $sync_token );
 
 $proplist = array();
@@ -82,11 +82,11 @@ if ( $sync_token == $new_token ) {
 else {
   $hide_older = '';
   if ( isset($c->hide_older_than) && intval($c->hide_older_than) > 0 )
-    $hide_older = " AND (CASE WHEN caldav_data.caldav_type<>'VEVENT' OR calendar_item.dtstart IS NULL THEN true ELSE calendar_item.dtstart > (now() - interval '".intval($c->hide_older_than)." days') END)";
+    $hide_older = " AND (CASE WHEN caldav_data.caldav_type<>'VEVENT' OR calendar_item.dtstart IS NULL OR calendar_item.rrule IS NOT NULL THEN true ELSE calendar_item.dtstart > (now() - interval '".intval($c->hide_older_than)." days') END)";
 
   $hide_todo = '';
   if ( isset($c->hide_TODO) && ($c->hide_TODO === true || (is_string($c->hide_TODO) && preg_match($c->hide_TODO, $_SERVER['HTTP_USER_AGENT']))) && ! $collection->HavePrivilegeTo('all') )
-    $hide_todo = " AND caldav_data.caldav_type NOT IN ('VTODO') ";
+    $hide_todo = " AND (caldav_data.caldav_type NOT IN ('VTODO') OR caldav_data.caldav_type IS NULL) ";
 
   if ( $sync_token == 0 ) {
     $sql = <<<EOSQL
@@ -117,10 +117,10 @@ EOSQL;
     }
   }
   $qry = new AwlQuery($sql, $params );
- 
+
   $last_dav_name = '';
   $first_status = 0;
-  
+
   if ( $qry->Exec("REPORT",__LINE__,__FILE__) ) {
     if ( $qry->rows() > 50 ) {
       // If there are more than 50 rows to send we should not send full data in response ...
@@ -144,7 +144,7 @@ EOSQL;
         else if ( $object->sync_status == 201 && $first_status == 404 ) {
           // ... Delete ... Create ... is indicated as a create, but don't forget we started with a delete
           array_pop($responses);
-          $dav_resource = new DAVResource($object);
+          $dav_resource = new DAVResource($object, $collection);
           $resultset = $dav_resource->GetPropStat($proplist,$reply);
           array_unshift($resultset, new XMLElement( 'href', ConstructURL($object->dav_name)));
           $responses[] = new XMLElement( 'response', $resultset );
@@ -167,7 +167,7 @@ EOSQL;
           $first_status = 404;
         }
         else {
-          $dav_resource = new DAVResource($object);
+          $dav_resource = new DAVResource($object, $collection);
           $resultset = $dav_resource->GetPropStat($proplist,$reply);
           array_unshift($resultset, new XMLElement( 'href', ConstructURL($object->dav_name)));
           $first_status = $object->sync_status;
