@@ -111,7 +111,7 @@ class ldapDriver
       //root to start search
       $this->baseDNUsers  = is_string($config['baseDNUsers']) ? array($config['baseDNUsers']) : $config['baseDNUsers'];
       $this->filterUsers  = (isset($config['filterUsers'])  ? $config['filterUsers']  : null);
-      $this->baseDNGroups = is_string($config['baseDNGroups']) ? array($config['baseDNGroups']) : $config['baseDNGroups'];
+      $this->baseDNGroups = (isset($config['baseDNGroups']) ? (is_string($config['baseDNGroups']) ? array($config['baseDNGroups']) : $config['baseDNGroups']) : null);
       $this->filterGroups = (isset($config['filterGroups']) ? $config['filterGroups'] : null);
   }
 
@@ -180,19 +180,22 @@ class ldapDriver
   }
 
   /**
-    * Returns the result of the LDAP query
+    * Actually look up a user in the LDAP directory
+    * (this is the LDAP part of LDAP_check() below)
     *
     * @param string $filter The filter used to search entries
     * @param array $attributes Attributes to be returned
+    * @param string $username username to check
     * @param string $passwd password to check
     * @return array Contains selected attributes from all entries corresponding to the given filter
     */
-  function requestUser( $filter, $attributes=NULL, $username, $passwd) {
+  function requestUser( $filter, $attributes, $username, $passwd) {
     global $c;
 
     $entry=NULL;
     // We get the DN of the USER
     $query = $this->ldap_query_one;
+    # ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, 7);
 
     foreach($this->baseDNUsers as $baseDNUsers) {
       $entry = $query($this->connect, $baseDNUsers, $filter, $attributes);
@@ -474,13 +477,19 @@ function sync_LDAP_groups(){
   }
 
   $ldap_groups = array_keys($ldap_groups_info);
+
   // users only in ldap
   $groups_to_create = array_diff($ldap_groups,$db_groups);
+
   // users only in db
   $groups_to_deactivate = array_diff($db_groups,$ldap_groups);
+
   // users present in ldap and in the db
   $groups_to_update = array_intersect($db_groups,$ldap_groups);
 
+  // groups where nothing was done
+  $groups_nothing_done[] = null;
+
   if ( sizeof ( $groups_to_create ) ){
     $validUserFields = awl_get_fields('usr');
     foreach ( $groups_to_create as $k => $group ){
