@@ -22,7 +22,7 @@ switch ( $request->path ) {
     $parameters = '';
     foreach( $_GET as $k => $v ) {
       $parameters .= ($parameters == '' ? '?' : '&' );
-      $parameters .= $k.'='.rawurlencode($v); 
+      $parameters .= $k.'='.rawurlencode($v);
     }
     header('Location: ' . $c->protocol_server_port . str_replace('/caldav.php', '', ConstructURL('/tz.php',true)).$parameters );
     $request->DoResponse(301); // Moved permanently
@@ -36,10 +36,15 @@ if ( $c->enable_scheduling != true )
  $request->DoResponse( 404, translate('The application program does not understand that request.') );
  // Does not return
 }
-dbg_log_array( 'well-known', 'method:'. $request->method );
+dbg_error_log( 'well-known', 'method: ' . $request->method );
 switch ( $request->method ) {
-  case 'GET':        ischedule_get();                      break;
-  case 'POST':       include('iSchedule-POST.php');             break;
+  case 'GET':
+    ischedule_get();
+    break;
+
+  case 'POST':
+    include('iSchedule-POST.php');
+    break;
 
   default:
     dbg_error_log( 'well-known', 'Unhandled request method >>%s<<', $request->method );
