@@ -70,8 +70,7 @@ function fetch_remote_list($base_url ) {
                 xml_get_current_line_number($xml_parser), xml_get_current_column_number($xml_parser) ));
   }
   xml_parser_free($xml_parser);
-  $position = 0;
-  return BuildXMLTree( $xml_tags, $position);
+  return BuildXMLTree( $xml_tags );
 }
   
 function fetch_remote_zone( $base_url, $tzid ) {
